@@ -24,7 +24,7 @@
  */
 
 /*!
- * @header IOAudioDevice
+ * @header IOAudioEngine
  */
 
 #ifndef _IOKIT_IOAUDIOENGINE_H
@@ -33,6 +33,7 @@
 #include <IOKit/IOService.h>
 
 #include <IOKit/audio/IOAudioTypes.h>
+#include <IOKit/IOBufferMemoryDescriptor.h>
 
 class OSDictionary;
 class OSCollection;
@@ -47,11 +48,12 @@ class IOCommandGate;
 
 /*!
  * @typedef IOAudioEnginePosition
- * @abstract Represents a position in an audio audio engine based on the sample frame within a 
+ * @abstract Represents a position in an audio audio engine.
+ * @discussion This position is based on the sample frame within a 
  *  loop around the sample buffer, and the loop count which starts at 0 when the audio engine
- *  begins playback
- * @field fSampleFrame The sample frame within the buffer - starts at 0
- * @field fLoopCount The number of times the ring buffer has looped
+ *  begins playback.
+ * @field fSampleFrame The sample frame within the buffer - starts at 0.
+ * @field fLoopCount The number of times the ring buffer has looped.
  */
 typedef struct {
     UInt32	fSampleFrame;
@@ -69,7 +71,7 @@ typedef struct {
  * @class IOAudioEngine
  * @abstract Abstract base class for a single audio audio / I/O engine.
  * @discussion An IOAudioEngine is defined by a single I/O engine to transfer data to
- *  or from one ore more sample buffers.  Each sample buffer is represented by a single IOAudioStream
+ *  or from one or more sample buffers.  Each sample buffer is represented by a single IOAudioStream
  *  instance.  A single IOAudioEngine must contain at least one IOAudioStream, but has no upper
  *  limit on the number of IOAudioStreams it may contain.  An IOAudioEngine instance may contain
  *  both input and output IOAudioStreams.
@@ -87,7 +89,7 @@ typedef struct {
  *  subclass must provide support for changing the hardware when a format or sample rate is
  *  changed.
  *
- *  There are several attributes associated with a single IOAudioEngine: .
+ *  There are several attributes associated with a single IOAudioEngine: 
  *
  *  The IOAudioEngine superclass provides a shared status buffer that contains all of the dynamic pieces
  *  of information about the audio engine (type IOAudioEngineStatus).  It runs an erase process on
@@ -150,54 +152,69 @@ public:
     /*! @var numSampleFramesPerBuffer */
     UInt32			numSampleFramesPerBuffer;
     
-    /*! @var sampleRate The current sample rate of the audio engine in samples per second */
+    /*! @var sampleRate 
+     *  The current sample rate of the audio engine in samples per second. */
     IOAudioSampleRate			sampleRate;
 
-    /*! @var numErasesPerBuffer The number of times the erase head get scheduled to run for each 
-     *   cycle of the audio engine */
+    /*! @var numErasesPerBuffer 
+     *  The number of times the erase head get scheduled to run for each 
+     *   cycle of the audio engine. */
     UInt32			numErasesPerBuffer;
-    /*! @var runEraseHead Set to true if the erase head is to run when the audio engine is running.  This is the
-    *    case if there are any output streams. */
+    /*! @var runEraseHead 
+     *  Set to true if the erase head is to run when the audio engine is running.  This is the case if there are any output streams. */
     bool			runEraseHead;
     
-    /*! @var audioEngineStopPosition When all clients have disconnected, this is set to one buffer length past the
+    /*! @var audioEngineStopPosition 
+     *  When all clients have disconnected, this is set to one buffer length past the
     *    current audio engine position at the time.  Then when the stop position is reached, the audio engine
     *    is stopped */
     IOAudioEnginePosition	audioEngineStopPosition;
 
-    /*! @var isRegistered Internal state variable to keep track or whether registerService() has been called. */
+    /*! @var isRegistered 
+     *  Internal state variable to keep track or whether registerService() has been called. */
     bool			isRegistered;
-    /*! @var configurationChangeInProgress Set to true after beginConfigurationChange() and false upon a 
+    /*! @var configurationChangeInProgress 
+     *  Set to true after beginConfigurationChange() and false upon a 
     *    subsequent call to completeConfigurationChange() or cancelConfigurationChange(). */
     bool			configurationChangeInProgress;
     
-    /*! @var state The current state of the IOAudioEngine - running, stopped, paused */
+    /*! @var state 
+     *  The current state of the IOAudioEngine - running, stopped, paused. */
     IOAudioEngineState		state;
 
-    /*! @var status Status struct shared with the CoreAudio.framework */
+    /*! @var status 
+     *  Status struct shared with the CoreAudio.framework. */
     IOAudioEngineStatus *	status;
 
-    /*! @var audioDevice The IOAudioDevice instance to which the IOAudioEngine belongs */
+    /*! @var audioDevice 
+     *  The IOAudioDevice instance to which the IOAudioEngine belongs. */
     IOAudioDevice *		audioDevice;
     
-    /*! @var workLoop The IOWorkLoop for the audio driver - shared with the IOAudioDevice */
+    /*! @var workLoop 
+     *  The IOWorkLoop for the audio driver - shared with the IOAudioDevice. */
     IOWorkLoop 			*workLoop;
-    /*! @var commandGate The IOCommandGate for this audio engine - attached to the driver's IOWorkLoop */
+    /*! @var commandGate 
+     *  The IOCommandGate for this audio engine - attached to the driver's IOWorkLoop. */
     IOCommandGate		*commandGate;
 
-    /*! @var inputStreams An OSSet of all of the input IOAudioStreams attached to this IOAudioEngine */
+    /*! @var inputStreams 
+     *  An OSSet of all of the input IOAudioStreams attached to this IOAudioEngine. */
     OSOrderedSet 	*inputStreams;
     UInt32			maxNumInputChannels;
-    /*! @var outputStreams An OSSet of all of the output IOAudioStreams attached to this IOAudioEngine */
+    /*! @var outputStreams 
+     *  An OSSet of all of the output IOAudioStreams attached to this IOAudioEngine. */
     OSOrderedSet	*outputStreams;
     UInt32			maxNumOutputChannels;
-    /*! @var userClients An OSSet of all of the currently connected user clients */
+    /*! @var userClients 
+     *  An OSSet of all of the currently connected user clients. */
     OSSet			*userClients;
-    /*! @var defaultAudioControls All of the IOAudioControls that affect this audio engine */
+    /*! @var defaultAudioControls 
+     *  All of the IOAudioControls that affect this audio engine. */
     OSSet			*defaultAudioControls;
     
-    /*! @var numActiveUserClients A total of the active user clients - those that are currently playing or 
-    *    recording audio */
+    /*! @var numActiveUserClients 
+     *  A total of the active user clients - those that are currently playing or 
+     *    recording audio. */
     UInt32			numActiveUserClients;
     UInt32			sampleOffset;
     
@@ -207,14 +224,18 @@ public:
 protected:
 
     /*!
-     * @var deviceStartedAudioEngine Used by the IOAudioDevice to determine responsibility for shutting
+     * @var deviceStartedAudioEngine 
+     *  Used by the IOAudioDevice to determine responsibility for shutting
      *  the audio engine down when it is no longer needed.
      */
     bool			deviceStartedAudioEngine;
     
 protected:
     struct ExpansionData {
-		UInt32		pauseCount;
+		UInt32								pauseCount;
+		IOBufferMemoryDescriptor			*statusDescriptor;
+		IOBufferMemoryDescriptor			*bytesInInputBufferArrayDescriptor;
+		IOBufferMemoryDescriptor			*bytesInOutputBufferArrayDescriptor;
 	};
     
     ExpansionData *reserved;
@@ -222,14 +243,20 @@ protected:
 public:
 // This takes the 0th reserved slot:
     virtual IOReturn performFormatChange(IOAudioStream *audioStream, const IOAudioStreamFormat *newFormat, const IOAudioStreamFormatExtension *formatExtension, const IOAudioSampleRate *newSampleRate);
+// This takes the 1st reserved slot:
+	virtual IOBufferMemoryDescriptor * getStatusDescriptor();
+// This takes the second reserved slot:
+	virtual IOReturn getNearestStartTime(IOAudioStream *audioStream, IOAudioTimeStamp *ioTimeStamp, bool isInput);
+	virtual IOBufferMemoryDescriptor * getBytesInInputBufferArrayDescriptor();
+	virtual IOBufferMemoryDescriptor * getBytesInOutputBufferArrayDescriptor();
 
 private:
     OSMetaClassDeclareReservedUsed(IOAudioEngine, 0);
+    OSMetaClassDeclareReservedUsed(IOAudioEngine, 1);
+    OSMetaClassDeclareReservedUsed(IOAudioEngine, 2);
+    OSMetaClassDeclareReservedUsed(IOAudioEngine, 3);
+    OSMetaClassDeclareReservedUsed(IOAudioEngine, 4);
 
-    OSMetaClassDeclareReservedUnused(IOAudioEngine, 1);
-    OSMetaClassDeclareReservedUnused(IOAudioEngine, 2);
-    OSMetaClassDeclareReservedUnused(IOAudioEngine, 3);
-    OSMetaClassDeclareReservedUnused(IOAudioEngine, 4);
     OSMetaClassDeclareReservedUnused(IOAudioEngine, 5);
     OSMetaClassDeclareReservedUnused(IOAudioEngine, 6);
     OSMetaClassDeclareReservedUnused(IOAudioEngine, 7);
@@ -277,19 +304,19 @@ private:
 public:
     /*!
      * @function createDictionaryFromSampleRate
-     * @abstract Internal routine used to generate a dictionary matching the given sample rate.
-     * @discussion Used to generate a sample rate dictionary for the IORegistry - used by the 
+     * @abstract Generates a dictionary matching the given sample rate.
+     * @discussion This is an internal routine used to generate a dictionary matching the given sample rate.  It is used to generate a sample rate dictionary for the I/O Registry - used by the 
      *  CoreAudio.framework.
-     * @result Returns the newly create OSDictionary
+     * @result Returns the newly create OSDictionary.
      */
     static OSDictionary *createDictionaryFromSampleRate(const IOAudioSampleRate *sampleRate, OSDictionary *rateDict = 0);
     
     /*!
      * @function createSampleRateFromDictionary
-     * @abstract Internal routine used to generate a sample rate from an OSDictionary.
-     * @discussion Used to generate a sample rate give a new OSDictionary from the IORegistry - coming
+     * @abstract Generates a sample rate from an OSDictionary.
+     * @discussion This is an internal routine used to generate a sample rate from an OSDictionary.  It is used to generate a sample rate give a new OSDictionary from the IORegistry - coming
      *  from the CoreAudio.framework.
-     * @result Returns the sample rate
+     * @result Returns the sample rate.
      */
     static IOAudioSampleRate *createSampleRateFromDictionary(const OSDictionary *rateDict, IOAudioSampleRate *sampleRate = 0);
 
@@ -301,8 +328,8 @@ public:
      *  values and allocates the shared status buffer.  Subclasses will likely want to override this method
      *  and do all of their common initialization in their implementation.  They do need to be sure to call
      *  IOAudioEngine's implementation of init and pay attention to the return value.
-     * @param properties The default properties for the IOAudioEngine
-     * @result Returns true if initialization was successful
+     * @param properties The default properties for the IOAudioEngine.
+     * @result Returns true if initialization was successful.
      */
     virtual bool init(OSDictionary *properties);
 
@@ -316,13 +343,13 @@ public:
 
     /*!
      * @function getWorkLoop
-     * @abstract Returns the IOWorkLoop for the driver
+     * @abstract Returns the IOWorkLoop for the driver.
      */
     virtual IOWorkLoop *getWorkLoop() const;
     
     /*!
      * @function getCommandGate
-     * @abstract Returns the IOCommandGate for this IOAudioEngine
+     * @abstract Returns the IOCommandGate for this IOAudioEngine.
      */
     virtual IOCommandGate *getCommandGate() const;
     
@@ -332,7 +359,7 @@ public:
      *  is the IOAudioDevice.
      * @discussion Subclasses will want to override start(IOService *, IOAudioDevice *) rather than this
      *  one.
-     * @param provider The service provider for the IOAudioEngine (the IOAudioDevice in this case)
+     * @param provider The service provider for the IOAudioEngine (the IOAudioDevice in this case).
      * @result Returns true if the IOAudioEngine was successfully started.
      */
     virtual bool start(IOService *provider);
@@ -346,9 +373,9 @@ public:
      *  all of the subclass-specific device initialization should be done.  Upon return from initHardware()
      *  all IOAudioStreams should be created and added to the audio engine.  Also, all IOAudioControls
      *  for this IOAudioEngine should be created and attached.
-     * @param provider The service provider for the IOAudioEngine
-     * @param device The IOAudioDevice to which this IOAudioEngine belongs
-     * @result Returns true if the service was successfully started
+     * @param provider The service provider for the IOAudioEngine.
+     * @param device The IOAudioDevice to which this IOAudioEngine belongs.
+     * @result Returns true if the service was successfully started.
      */
     virtual bool start(IOService *provider, IOAudioDevice *device);
     
@@ -359,25 +386,25 @@ public:
      * @discussion Upon return from this function, all IOAudioStreams and IOAudioControls should be created
      *  and the audio engine should be ready to be started when a client requests that playback begin.
      * @function provider The service provider numb for this audio engine - typically the IOAudioDevice.
-     * @result Returns true if the hardware was successfully initialized
+     * @result Returns true if the hardware was successfully initialized.
      */
     virtual bool initHardware(IOService *provider);
 
     /*!
      * @function stop
-     * @abstract Called to stop the service and prepare for the driver to be terminated.
+     * @abstract Stops the service and prepares for the driver to be terminated.
      * @discussion This function is called before the driver is terminated and usually means that the device
      *  has been removed from the system.
-     * @param provider The service provider for the IOAudioEngine
+     * @param provider The service provider for the IOAudioEngine.
      */
     virtual void stop(IOService *provider);
 
     /*!
      * @function registerService
-     * @abstract Called when this audio engine is ready to begin vending services
+     * @abstract Called when this audio engine is ready to begin vending services.
      * @discussion This function is called by IOAudioDevice::activateAudioEngine() once the audio engine
      *  has been fully initialized and is ready to begin audio playback.
-     * @var options
+     * @param options
      */
     virtual void registerService(IOOptionBits options = 0);
     
@@ -388,25 +415,25 @@ public:
 
     /*!
      * @function newUserClient
-     * @abstract Called to request a new user client object for this service
-     * @discussion This function is called automatically by IOKit when a user process attempts
+     * @abstract Requests a new user client object for this service.
+     * @discussion This function is called automatically by I/O Kit when a user process attempts
      *  to connect to this service.  It allocates a new IOAudioEngineUserClient object and increments
      *  the number of connections for this audio engine.  If this is the first user client for this IOAudioEngine,
      *  it calls startAudioEngine().  There is no need to call this function directly.
-     * @param task The task requesting the new user client
-     * @param securityID Optional security paramater passed in by the client - ignored
-     * @param type Optional user client type passed in by the client - ignored
-     * @param handler The new IOUserClient * must be stored in this param on a successful completion
+     * @param task The task requesting the new user client.
+     * @param securityID Optional security paramater passed in by the client - ignored.
+     * @param type Optional user client type passed in by the client - ignored.
+     * @param handler The new IOUserClient * must be stored in this param on a successful completion.
      * @result Returns kIOReturnSuccess on success.  May also result kIOReturnError or kIOReturnNoMemory.
      */
     virtual IOReturn newUserClient(task_t task, void *securityID, UInt32 type, IOUserClient **handler);
 
     /*!
      * @function addAudioStream
-     * @abstract Called by the driver to add an IOAudioStream to the audio engine
-     * @discussion This must be called at least once to make sure the audio engine has at least one IOAudioStream.
-     * @param stream The IOAudioStream to be added
-     * @result Returns kIOReturnSuccess if the stream was successfully added
+     * @abstract Adds an IOAudioStream to the audio engine.
+     * @discussion This function is called by the driver to add an IOAudioStream to the audio engine.  This must be called at least once to make sure the audio engine has at least one IOAudioStream.
+     * @param stream The IOAudioStream to be added.
+     * @result Returns kIOReturnSuccess if the stream was successfully added.
      */
     virtual IOReturn addAudioStream(IOAudioStream *stream);
     
@@ -419,7 +446,7 @@ public:
 
     /*!
      * @function resetStatusBuffer
-     * @abstract Resets the status buffer to its default values
+     * @abstract Resets the status buffer to its default values.
      * @discussion This is called during startAudioEngine() and resumeAudioEngine() to clear out the status buffer
      *  in preparation of starting up the I/O engine.  There is no need to call this directly.
      */
@@ -435,7 +462,7 @@ public:
     
     /*!
      * @function getCurrentSampleFrame
-     * @abstract Called to get the current sample frame from the IOAudioEngine subclass
+     * @abstract Gets the current sample frame from the IOAudioEngine subclass.
      * @discussion
      * @result
      */
@@ -443,12 +470,12 @@ public:
 
     /*!
      * @function startAudioEngine
-     * @abstract Called to start the audio I/O engine
+     * @abstract Starts the audio I/O engine.
      * @discussion This method is called automatically when the audio engine is placed into use the first time.
-     *  This must be overridden by the subclass.  No call to the superclass' implementation is
-     *  necessary.  The subclass' implementation must start up the audio I/O engine.  This includes any audio
+     *  This must be overridden by the subclass.  No call to the superclass's implementation is
+     *  necessary.  The subclass's implementation must start up the audio I/O engine.  This includes any audio
      *  engine that needs to be started as well as any interrupts that need to be enabled.  Upon successfully
-     *  starting the engine, the subclass' implementation must call _setState(kAudioEngineRunning).  If
+     *  starting the engine, the subclass's implementation must call _setState(kAudioEngineRunning).  If
      *  it has also checked the state using getState() earlier in the implementation, the stateLock must be
      *  acquired for the entire initialization process (using IORecursiveLockLock(stateLock) and
      *  IORecursiveLockUnlock(stateLock)) to ensure that the state remains consistent.  See the general class
@@ -459,10 +486,10 @@ public:
 
     /*!
      * @function stopAudioEngine
-     * @abstract Called to stop the audio I/O engine
+     * @abstract Stops the audio I/O engine.
      * @discussion This method is called automatically when the last client disconnects from this audio engine.
-     *  It must be overridden by the subclass.  No call to the superclass' implementation is necessary.
-     *  The subclass' implementation must stop the audio I/O engine.  The audio engine (if it exists) should
+     *  It must be overridden by the subclass.  No call to the superclass's implementation is necessary.
+     *  The subclass's implementation must stop the audio I/O engine.  The audio engine (if it exists) should
      *  be stopped and any interrupts disabled.  Upon successfully stopping the engine, the subclass must call
      *  _setState(kAudioEngineStopped).  If it has also checked the state using getState() earlier in the
      *  implementation, the stateLock must be acquired for the entire initialization process (using
@@ -498,66 +525,69 @@ public:
 
     /*! 
      * @function getState
-     * @abstract Returns the current state of the IOAudioEngine
+     * @abstract Returns the current state of the IOAudioEngine.
      * @discussion If this method is called in preparation for calling _setState(), the stateLock must
      *  be acquired before the first call to getState() and held until after the last call to _setState().
      *  Be careful not to return from the code acquiring the lock while the lock is being held.  That
      *  will cause a deadlock situation.
-     * @result The current state of the IOAudioEngine: kAudioEngineRunning, kAudioEngineStopped
+     * @result The current state of the IOAudioEngine: kAudioEngineRunning, kAudioEngineStopped.
      */
     virtual IOAudioEngineState getState();
 
     /*!
-     * @function getSampleRate Returns the sample rate of the IOAudioEngine in samples per second
+     * @function getSampleRate 
+     * @abstract Returns the sample rate of the IOAudioEngine in samples per second.
      */
     virtual const IOAudioSampleRate *getSampleRate();
     
     virtual IOReturn hardwareSampleRateChanged(const IOAudioSampleRate *sampleRate);
 
     /*!
-     * @function getErases Returns true if the audio engine will run the erase head when the audio engine is running.
+     * @function getErases 
+     * @abstract Returns true if the audio engine will run the erase head when the audio engine is running.
      */
     virtual bool getRunEraseHead();
 
     /*!
-     * @function getStatus Returns a pointer to the shared status buffer
+     * @function getStatus 
+     * @abstract Returns a pointer to the shared status buffer.
      */
     virtual const IOAudioEngineStatus *getStatus();
 
     /*!
      * @function timerCallback
-     * @abstract A static method used as a callback for the IOAudioDevice timer services
+     * @abstract A static method used as a callback for the IOAudioDevice timer services.
      * @discussion This method implements the IOAudioDevice::TimerEvent type.
-     * @param arg1 The IOAudioEngine that is the target of the event
-     * @param device The IOAudioDevice that sent the timer event
+     * @param arg1 The IOAudioEngine that is the target of the event.
+     * @param device The IOAudioDevice that sent the timer event.
      */
     static void timerCallback(OSObject *arg1, IOAudioDevice *device);
 
     /*!
      * @function timerFired
-     * @abstract Called by timerCallback to indicate the timer has fired
-     * @discussion This method calls performErase() and performFlush() to do erase head processing and
+     * @abstract Indicates the timer has fired.
+     * @discussion This method is called by timerCallback to indicate the timer has fired.  This method calls performErase() and performFlush() to do erase head processing and
      *  audio engine flushing each time the timer event fires.
      */
     virtual void timerFired();
 
     /*!
      * @function getTimerInterval
-     * @abstract Called to get the timer interval for use by the timer event
-     * @discussion This method is called each timer the timer event is enabled through addTimer().  The default
+     * @abstract Gets the timer interval for use by the timer event.
+     * @discussion This method is called each time the timer event is enabled through addTimer().  The default
      *  implementation is set to return a value such that the timer event runs n times each cycle of the audio
      *  engine through the sample buffer.  The value n is stored as the instance variable: numErasesPerBuffer.
      *  The default value of numErasesPerBuffer is set to IOAUDIOENGINE_DEFAULT_NUM_ERASES_PER_BUFFER which is 4.
      *  A subclass may change the value of numErasesPerBuffer or override getTimerInterval.  If it is overridden,
-     *  the subclass should call the superclass' implementation, compare its interval with the superclass' and
+     *  the subclass should call the superclass's implementation, compare its interval with the superclass's and
      *  return the smaller of the two.
-     * @result Returns the interval for the timer event
+     * @result Returns the interval for the timer event.
      */
     virtual AbsoluteTime getTimerInterval();
 
     /*!
      * @function performErase
-     * @abstract Called to perform erase head processing
+     * @abstract Performs erase head processing.
      * @discussion This method is called automatically each time the timer event fires and erases the sample
      *  buffer and mix buffer from the previous location up to the current location of the audio engine.
      */
@@ -565,7 +595,7 @@ public:
 
     /*!
      * @function performFlush
-     * @abstract Called to perform the flush operation
+     * @abstract Performs the flush operation.
      * @discussion This method is called automatically each time the timer event fires.  It stops the audio engine
      *  if there are no more clients and the audio engine is passed the latest flush ending position.
      */
@@ -600,8 +630,8 @@ protected:
 
     /*!
      * @function initKeys
-     * @abstract Internal initialization routine which generates the OSSymbols with the keys.
-     * @discussion Do not call this directly.
+     * @abstract Generates the OSSymbols with the keys.
+     * @discussion Do not call this directly.  This is an internal initialization routine.
      */ 
     static void initKeys();
     
@@ -610,29 +640,29 @@ protected:
 
     /*!
      * @function _setState
-     * @abstract Used to indicate that the audio engine is in the specified state
+     * @abstract Indicates that the audio engine is in the specified state.
      * @discussion This method simply sets the internal state of the audio engine to the specified state.  It does not
      *  affect a change to the state.  It does however keep other internal state-related attributes consistent.
      *  For example, it enables or disables the timer as needed when the state changes to running or stopped.
-     * @param newState The state the audio engine is in
-     * @result Returns the old state
+     * @param newState The state the audio engine is in.
+     * @result Returns the old state.
      */
     virtual IOAudioEngineState setState(IOAudioEngineState newState);
 
     /*!
      * @function setSampleRate
-     * @abstract Records the sample rate of the audio engine
+     * @abstract Records the sample rate of the audio engine.
      * @discussion  This method must be called during initialization of a new audio engine to record the audio engine's
      *  initial sample rate.  It also is intended to be used to record changes to the sample rate during use.
      *  Currently changing sample rates after the audio engine has been started is not supported.
      *  It may require that the sample buffers be re-sized.  This will be available in an upcoming release.
-     * @param newSampleRate The sample rate of the audio engine in samples per second
+     * @param newSampleRate The sample rate of the audio engine in samples per second.
      */
     virtual void setSampleRate(const IOAudioSampleRate *newSampleRate);
 
     /*!
      * @function _setSampleLatency
-     * @abstract Sets the sample latency for the audio engine
+     * @abstract Sets the sample latency for the audio engine.
      * @discussion The sample latency represents the number of samples ahead of the playback head
      *  that it is safe to write into the sample buffer.  The audio device API will never write
      *  closer to the playback head than the number of samples specified.  For input audio engines
@@ -645,16 +675,16 @@ protected:
 
     /*!
      * @function _setErases
-     * @abstract Tells the audio engine whether or not to run the erase head
+     * @abstract Tells the audio engine whether or not to run the erase head.
      * @discussion By default, output audio engines run the erase head and input audio engines do not.  This method can
      *  be called after _setDirection() is called in order to change the default behavior.
-     * @param runEraseHead The audio engine will run the erase head if this value is true
+     * @param runEraseHead The audio engine will run the erase head if this value is true.
      */
     virtual void setRunEraseHead(bool runEraseHead);
 
     /*!
      * @function clientClosed
-     * @abstract Called automatically when a user client closes its connection to the audio engine
+     * @abstract Called automatically when a user client closes its connection to the audio engine.
      * @discussion This method decrements the number of connections to the audio engine and if they reach
      *  zero, the audio engine is called with a call to stopAudioEngine().  This method should not be called directly.
      * @param client The user client that has disconnected.
@@ -663,7 +693,7 @@ protected:
 
     /*!
      * @function addTimer
-     * @abstract Enables the timer event for the audio engine
+     * @abstract Enables the timer event for the audio engine.
      * @discussion There is a timer event needed by the IOAudioEngine for processing the erase head
      *  and performing flushing operations. When the timer fires, the method timerFired() is ultimately
      *  called which in turn calls performErase() and performFlush().  This is called automatically
@@ -675,7 +705,7 @@ protected:
 
     /*!
      * @function removeTimer
-     * @abstract Disables the timer event for the audio engine
+     * @abstract Disables the timer event for the audio engine.
      * @discussion  This method is called automatically to disable the timer event for this audio engine.
      *  There is need to call it directly.  This method is called by _setState() when the audio engine state
      *  is changed from kAudioEngineRunning to one of the stopped states.
