@@ -71,8 +71,8 @@ OSMetaClassDefineReservedUsed(IOAudioStream, 4);
 OSMetaClassDefineReservedUsed(IOAudioStream, 5);
 OSMetaClassDefineReservedUsed(IOAudioStream, 6);
 OSMetaClassDefineReservedUsed(IOAudioStream, 7);
+OSMetaClassDefineReservedUsed(IOAudioStream, 8);
 
-OSMetaClassDefineReservedUnused(IOAudioStream, 8);
 OSMetaClassDefineReservedUnused(IOAudioStream, 9);
 OSMetaClassDefineReservedUnused(IOAudioStream, 10);
 OSMetaClassDefineReservedUnused(IOAudioStream, 11);
@@ -453,6 +453,14 @@ IOReturn IOAudioStream::mixOutputSamples(const void *sourceBuf, void *mixBuf, UI
 	return kIOReturnSuccess;
 }
 
+void IOAudioStream::setSampleLatency(UInt32 numSamples)
+{
+#ifdef DEBUG_CALLS
+    IOLog("IOAudioStream[%p]::setSampleLatency(0x%lx)\n", this, numSamples);
+#endif
+    setProperty(kIOAudioStreamSampleLatencyKey, numSamples, sizeof(UInt32)*8);
+}
+
 // Original code from here on:
 const OSSymbol *IOAudioStream::gDirectionKey = NULL;
 const OSSymbol *IOAudioStream::gNumChannelsKey = NULL;
@@ -1772,7 +1780,7 @@ IOReturn IOAudioStream::addDefaultAudioControl(IOAudioControl *defaultAudioContr
             
             if (defaultAudioControl->attachAndStart(this)) {
                 if (!defaultAudioControls) {
-                    defaultAudioControls = OSSet::withObjects(&(const OSObject *)defaultAudioControl, 1, 1);
+                    defaultAudioControls = OSSet::withObjects((const OSObject **)&defaultAudioControl, 1, 1);
                 } else {
                     defaultAudioControls->setObject(defaultAudioControl);
                 }
