@@ -23,7 +23,7 @@
  * @APPLE_LICENSE_HEADER_END@
  */  
 
-#ifndef _IOKIT_IOAUDIOTOGGLECONTROl_H
+#ifndef _IOKIT_IOAUDIOTOGGLECONTROL_H
 #define _IOKIT_IOAUDIOTOGGLECONTROL_H
 
 #include <IOKit/audio/IOAudioControl.h>
